@@ -31,6 +31,7 @@ class DecoderConfig:
     num_cross_attention_qk_channels: Optional[int] = None
     num_cross_attention_v_channels: Optional[int] = None
     cross_attention_widening_factor: int = 1
+    cross_attention_residual: bool = True
     dropout: float = 0.0
     init_scale: float = 0.02
     freeze: bool = False
@@ -51,6 +52,7 @@ class PerceiverConfig(Generic[E, D]):
     num_latent_channels: int
     activation_checkpointing: bool = False
     activation_offloading: bool = False
+    params: Optional[str] = None
 
 
 @dataclass
